#include "twi.h"  //Библиотека для I2C
#include "FS.h"     // Библиотека для SPIFFS
#include <ESP8266WiFi.h>                                // Подключаем библиотеку ESP8266WiFi
#include <ESP8266HTTPClient.h>                          // Подключаем библиотеку ESP8266HTTPClient

float busvoltage1 = 0, shuntvoltage1=0, loadvoltage1=0 ;       // Переменные для получения значений напряжения на шине, шунте и нагрузке с датчиков INA219
float current_mA1 = 0, current_mA2 = 0, current_mA3 = 0;       // Переменные для получения значений тока с датчиков INA219 

float _current_lsb = 0.0;           // LSB для тока
float _power_lsb = 0.0;             // LSB для мощности

uint16_t _cal_value = 0;            // Калибровочное значение

const float _r_shunt = 0.1f, _i_max = 3.2f; //Калибровачные значения INA219
const uint8_t _iic_address1 = 0x40, _iic_address2 = 0x41, _iic_address3 = 0x44,_i2c_address = 0x76; //Адреса датчиков BME280, INA219 1-3

float hum=0,pressure=0,temp=0,dew_point=0,rosa=0;  //Переменные для показаний с датчика BME280
bool t=true,h=true,p=true;  //Переменные для проверки того, что данные с датчика BME280 получены
int8_t ret; //Переменная для проверки того, что данные с датчика BME280 получены

struct {                        // Структура для калибровочных значений BME280
        uint16_t _T1;
        int16_t _T2;
        int16_t _T3;
        uint16_t _P1;
        int16_t _P2;
        int16_t _P3;
        int16_t _P4;
        int16_t _P5;
        int16_t _P6;
        int16_t _P7;
        int16_t _P8;
        int16_t _P9;
        uint8_t _H1;
        int16_t _H2;
        uint8_t _H3;
        int16_t _H4;
        int16_t _H5;
        int8_t _H6;
    } CalibrationData;

//Системные переменные и значения датчика BME280
uint8_t _operating_mode = 0x02;    // Принудительный режим работы
uint8_t _standby_time = 0x03;    // Time between measurements in NORMAL_MODE   
uint8_t _filter_coef = 0x04;    // Filter ratio IIR                           
uint8_t _temp_oversampl = 0x03; // Temperature module oversampling parameter  
uint8_t _hum_oversampl = 0x01;  // Humidity module oversampling parameter     
uint8_t _press_oversampl = 0x02;  // Pressure module oversampling parameter    

String json;                              // Переменная JSON для отправки данных на сервер
String whoami;                            // Имя устройства
String Akey;                              // NeKKxx1
String wifi_name;                         // Название WiFi сети
String wifi_pass;                         // Пароль WiFi сети
IPAddress myIP;                           // Хранит IP адресс
HTTPClient http;
WiFiClient wifiClient;

static int default_sda_pin = SDA;  //дефолтный пин SDA
static int default_scl_pin = SCL;  //дефолтный пин SCL

uint8_t rxBuffer[128]; //буфер для отправки данных по I2C
uint8_t txBuffer[128]; //буфер для чтения данных по I2C

//индексы и размеры буферов
size_t  rxBufferIndex  = 0;
size_t  rxBufferLength = 0;
size_t  txBufferIndex  = 0;
size_t  txBufferLength = 0;

uint16_t cfg_register_sleep;

void sendJsonToDB()  //отправка данных в БД
{
  json  =  "{\"system\":{ ";
  json += "\"Akey\":\""+Akey+"\", ";
  json += "\"Serial\":\""+whoami+"\", ";
  json += "\"Version\":\"2023-05-11\", ";
  json += "\"RSSI\":\"" + String(WiFi.RSSI()) + "\",";
  json += "\"MAC\":\"" + String(WiFi.macAddress()) + "\",";
  myIP=WiFi.localIP();
  json += "\"IP\":\""; json+= (String)myIP[0] + String(".") + (String)myIP[1] + String(".") +(String)myIP[2] + String(".") + (String)myIP[3]; json += "\"},";
  json += "\"BME280\":{ ";
  json += "\"temp\":"+String(temp)+",";
  json += "\"humidity\":"+String(hum)+",";
  json += "\"dew_point\":"+String(dew_point)+",";
  json += "\"pressure\":"+String(pressure)+"},";
  json += "\"INA219\":{ ";
  json += "\"Current_cons\":"+String(current_mA1)+",";
  json += "\"Voltage_load\":"+String(loadvoltage1)+",";
  json += "\"Current_bat1\":"+String(current_mA2)+",";
  json += "\"Current_bat2\":"+String(current_mA3)+"} }";

//http.begin(wifiClient,"http://188.35.161.31/core/jsonadd.php");
  http.begin(wifiClient,"http://dbrobo.mgul.ac.ru/core/jsonadd.php");
  http.addHeader("Content-Type", "application/json");
  int resulthttp = http.POST(json);
  Serial.print("Done: ");
  Serial.println(resulthttp);
  http.end();
}


//Чтение из буфера I2C
int Wread(){ 
  int value = -1;
    if (rxBufferIndex < rxBufferLength)
    {
        value = rxBuffer[rxBufferIndex];
        ++rxBufferIndex;
    }
  return value;
}

void setup() {
  Serial.begin(9600);

  //рассчет калибровочных значений INA219
    _current_lsb = _i_max / 32768.0f;                           // Рассчет LSB для тока (см. доку INA219)
    _power_lsb = _current_lsb * 20.0f;                          // Рассчет LSB для мощности (см. доку INA219)
    _cal_value = trunc(0.04096f / (_current_lsb * _r_shunt));   // Рассчет калибровочного значения (см. доку INA219)
      
    uint16_t cfg_register = ( 0x8000 << 8 | 0x8000) & ~(0b11 << 11);;      // Клеим и возвращаем результат   
    uint16_t vshunt_max_mv = (_r_shunt * _i_max) * 1000;        // Считаем макс напряжение на шунте при макс ожидаемом токе

    if (vshunt_max_mv <= 40) cfg_register |= 0b00 << 11;        // +/- 40mV  - выбираем диапазон
    else if (vshunt_max_mv <= 80) cfg_register |= 0b01 << 11;   // +/- 80mV
    else if (vshunt_max_mv <= 160) cfg_register |= 0b10 << 11;  // +/- 160mV
    else cfg_register |= 0b11 << 11;                            // +/- 320mV  

//инициализация интерфейса I2C  
  twi_init(default_sda_pin, default_scl_pin); 

//Пробуждение INA219
    txBuffer[txBufferIndex] = 0x00;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);     
    txBufferIndex  = 0;
    txBufferLength = 0;

    rxBufferLength = (twi_readFrom(_iic_address1, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;

    cfg_register_sleep = (Wread() << 8 | Wread()) & ~(0b111);      // Прочитать конф. регистр и стереть биты режима

    txBufferIndex  = 0;
    txBufferLength = 0;

    txBuffer[txBufferIndex] = 0x00;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    txBuffer[txBufferIndex] = highByte(cfg_register_sleep | (false ? 0b000 : 0b111));
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    txBuffer[txBufferIndex] = lowByte(cfg_register_sleep | (false ? 0b000 : 0b111));
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true); 
    
    txBufferIndex  = 0;
    txBufferLength = 0;
 
//инициализания BME280 ч.1 
    txBuffer[txBufferIndex] = 0x0E;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = 0xB6;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;
         
    txBuffer[txBufferIndex] = 0xD0;
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 
    
    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;
    
    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)1, true) == 0) ? 1 : 0;
    rxBufferIndex  = 0;
    
        
// получение расчетных значений BME280 первая часть (ч.1)
    rxBufferLength = 0;
    txBuffer[txBufferIndex] = 0x88;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;
    
    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;


//сброс датчиков INA219
    txBufferIndex  = 0;
    txBufferLength = 0;
    
    txBuffer[txBufferIndex] = 0x05;
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    txBuffer[txBufferIndex] = highByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = lowByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true);
    
    txBufferIndex  = 0;
    txBufferLength = 0;     

// получение расчетных значений BME280 первая часть (ч.2)
    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)25, true) == 0) ? 25 : 0;
    rxBufferIndex  = 0;
    
    // чтение
    CalibrationData._T1 = (Wread() | (Wread() << 8));
    CalibrationData._T2 = (Wread() | (Wread() << 8));
    CalibrationData._T3 = (Wread() | (Wread() << 8));
    CalibrationData._P1 = (Wread() | (Wread() << 8));
    CalibrationData._P2 = (Wread() | (Wread() << 8));
    CalibrationData._P3 = (Wread() | (Wread() << 8));
    CalibrationData._P4 = (Wread() | (Wread() << 8));
    CalibrationData._P5 = (Wread() | (Wread() << 8));
    CalibrationData._P6 = (Wread() | (Wread() << 8));
    CalibrationData._P7 = (Wread() | (Wread() << 8));
    CalibrationData._P8 = (Wread() | (Wread() << 8));
    CalibrationData._P9 = (Wread() | (Wread() << 8));
    CalibrationData._H1 = Wread();
    
// получение расчетных значений BME280 вторая часть

    rxBufferLength = 0;
    txBuffer[txBufferIndex] = 0xE1;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;
    
    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;

//установка калибровочного значения INA219
    txBuffer[txBufferIndex] = 0x00;
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    txBuffer[txBufferIndex] = highByte(cfg_register);
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = lowByte(cfg_register);
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true);
    
    txBufferIndex  = 0;
    txBufferLength = 0; 
        
//чтение данных с BME280    
    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)8, true) == 0) ? 8 : 0;
    rxBufferIndex  = 0;
    
    // чтение
    CalibrationData._H2 = (Wread() | (Wread() << 8));
    CalibrationData._H3 = Wread();
    CalibrationData._H4 = (Wread() << 4);
    uint8_t interVal = Wread();
    CalibrationData._H4 |= (interVal & 0xF);
    CalibrationData._H5 = (((interVal & 0xF0) >> 4) | (Wread() << 4));
    CalibrationData._H6 = Wread();

//Установка калибровочных значений BME280

    txBufferIndex  = 0;
    txBufferLength = 0;
    
     txBuffer[txBufferIndex] = 0xF2;
     ++txBufferIndex;
     txBufferLength = txBufferIndex; 

     txBuffer[txBufferIndex] = _hum_oversampl;
     ++txBufferIndex;
     txBufferLength = txBufferIndex; 

     twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
     txBufferIndex  = 0;
     txBufferLength = 0;

    
    
    txBuffer[txBufferIndex] = 0xF2;
     ++txBufferIndex;
     txBufferLength = txBufferIndex; 

    
    
     txBufferIndex  = 0;
     txBufferLength = 0;
    
     txBuffer[txBufferIndex] = 0xF2;
     ++txBufferIndex;
     txBufferLength = txBufferIndex; 

     twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
     txBufferIndex  = 0;
     txBufferLength = 0;

//напряжение на шунте
    txBuffer[txBufferIndex] = 0x05;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = highByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex;     

    txBuffer[txBufferIndex] = lowByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;    
    
//напряжение на шунте
    txBuffer[txBufferIndex] = 0x01;
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;

    rxBufferLength = (twi_readFrom(_iic_address1, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;

    int16_t value1 = Wread() << 8 | Wread();      // Клеим и возвращаем результат          
         
//продолжение калибровки BME280
     rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)1, true) == 0) ? 1 : 0;
     rxBufferIndex  = 0;
     

    txBuffer[txBufferIndex] = Wread();
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;  

         
        
    txBuffer[txBufferIndex] = 0xF4;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = ((_temp_oversampl << 5) | (_press_oversampl << 2) | _operating_mode);
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;    
    

    
    txBuffer[txBufferIndex] = 0xF5;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = ((_standby_time << 5) | (_filter_coef << 2));
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

// Чтение напряжения 
    txBufferIndex  = 0;
    txBufferLength = 0; 
    
    txBuffer[txBufferIndex] = 0x02;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;    

    rxBufferLength = (twi_readFrom(_iic_address1, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;

    uint16_t value2 =  Wread() << 8 | Wread();      // Клеим и возвращаем результат   

//продолжение калибровки BME280    

    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;                      
        
// Чтение тока
    txBufferIndex  = 0;
    txBufferLength = 0; 
    
    txBuffer[txBufferIndex] = 0x05;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    txBuffer[txBufferIndex] = highByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex;     

    txBuffer[txBufferIndex] = lowByte(_cal_value);
    ++txBufferIndex;
    txBufferLength = txBufferIndex; 

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true);
    
    txBufferIndex  = 0;
    txBufferLength = 0;
     
//чтение температуры

    txBufferIndex  = 0;
    txBufferLength = 0;   
    
    txBuffer[txBufferIndex] = 0xFA;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    ret     = twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;     
         

//продолжение чтения тока

    txBuffer[txBufferIndex] = 0x04;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true);
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true);

//продолжение чтения температуры

    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)3, true) == 0) ? 3 : 0;
    rxBufferIndex  = 0;
    
    if (ret != 0) t=false;    
    int32_t temp_raw = (((uint32_t)Wread() << 16) | ((uint32_t)Wread() << 8) | (uint32_t)Wread());    

//продолжение чтения тока    
    
    txBufferIndex  = 0;
    txBufferLength = 0;     
     
    rxBufferLength = (twi_readFrom(_iic_address1, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;
    int16_t value3 =  Wread() << 8 | Wread();      // Клеим и возвращаем результат    


//давление

    txBufferIndex  = 0;
    txBufferLength = 0;   
    
    txBuffer[txBufferIndex] = 0xF7;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    ret     = twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;

//продолжение чтения тока        

    txBufferIndex  = 0;
    txBufferLength = 0;     
     
    rxBufferLength = (twi_readFrom(_iic_address2, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;
    int16_t value4 =  Wread() << 8 | Wread();      // Клеим и возвращаем результат    

//продолжение чтения давления     
    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)3, true) == 0) ? 3 : 0;
    rxBufferIndex  = 0;
    
    if (ret != 0) p=false; 
    uint32_t press_raw = (((uint32_t)Wread() << 16) | ((uint32_t)Wread() << 8) | (uint32_t)Wread()); 

//влажность

    txBufferIndex  = 0;
    txBufferLength = 0;   
    
    txBuffer[txBufferIndex] = 0xFD;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;  

    twi_writeTo(_i2c_address, txBuffer, txBufferLength, true);
    txBufferIndex  = 0;
    txBufferLength = 0;           

//продолжение чтения тока                

    txBufferIndex  = 0;
    txBufferLength = 0;     
     
    rxBufferLength = (twi_readFrom(_iic_address3, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;
    int16_t value5 =  Wread() << 8 | Wread();      // Клеим и возвращаем результат                          
     
//продолжение чтения влажности    
     
    rxBufferLength = (twi_readFrom(_i2c_address, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;
   
    int32_t hum_raw = ((uint16_t)Wread() << 8) | (uint16_t)Wread(); // Read humidity data 
    if (hum_raw == 0x8000) h=false;

//Погружаем INA219 в сон
    txBuffer[txBufferIndex] = 0x00;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);     
    txBufferIndex  = 0;
    txBufferLength = 0;

    rxBufferLength = (twi_readFrom(_iic_address1, rxBuffer, (uint8_t)2, true) == 0) ? 2 : 0;
    rxBufferIndex  = 0;

    cfg_register_sleep = (Wread() << 8 | Wread()) & ~(0b111);      // Прочитать конф. регистр и стереть биты 

    txBufferIndex  = 0;
    txBufferLength = 0;

    txBuffer[txBufferIndex] = 0x00;
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    txBuffer[txBufferIndex] = highByte(cfg_register_sleep | (true ? 0b000 : 0b111));
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    txBuffer[txBufferIndex] = lowByte(cfg_register_sleep | (true ? 0b000 : 0b111));
    ++txBufferIndex;
    txBufferLength = txBufferIndex;

    twi_writeTo(_iic_address1, txBuffer, txBufferLength, true);                         
    twi_writeTo(_iic_address2, txBuffer, txBufferLength, true); 
    twi_writeTo(_iic_address3, txBuffer, txBufferLength, true); 

    

//преобразуем полученные значения 
    current_mA1 = value3 * _current_lsb*1000;                            // LSB рассчитывается на основе макс. ожидаемого тока, умножаем и возвращаем  
    current_mA2 = value4 * _current_lsb*1000;                            // LSB рассчитывается на основе макс. ожидаемого тока, умножаем и возвращаем
    current_mA3 = value5 * _current_lsb*1000;                            // LSB рассчитывается на основе макс. ожидаемого тока, умножаем и возвращаем
    busvoltage1 =(value2 >> 3) * 0.004f;                           // LSB = 4mV = 0.004V, Сдвигаем значение до 12 бит и умножаем 
    shuntvoltage1= value1 * 0.00001f;                                // LSB = 10uV = 0.00001V, умножаем и возвращаем
    loadvoltage1 = busvoltage1 + (shuntvoltage1 / 1000); // Расчет напряжение на нагрузки
       
    if (t) {
      temp_raw >>= 4;                         // Start temperature reading in integers
      int32_t value_1 = ((((temp_raw >> 3) - ((int32_t)CalibrationData._T1 << 1))) *
      ((int32_t)CalibrationData._T2)) >> 11;
      int32_t value_2 = (((((temp_raw >> 4) - ((int32_t)CalibrationData._T1)) * 
      ((temp_raw >> 4) - ((int32_t)CalibrationData._T1))) >> 12) * ((int32_t)CalibrationData._T3)) >> 14;

      temp_raw = ((int32_t)value_1 + value_2);              // Return temperature in integers
    
      float T = (temp_raw * 5 + 128) >> 8;
      temp= T / 100.0; 
    }else{temp=0;}

    if (p) {
      press_raw >>= 4;                        // Start pressure converting
      int64_t value_1 = ((int64_t)temp_raw) - 128000;
      int64_t value_2 = value_1 * value_1 * (int64_t)CalibrationData._P6;
      value_2 = value_2 + ((value_1 * (int64_t)CalibrationData._P5) << 17);
      value_2 = value_2 + (((int64_t)CalibrationData._P4) << 35);
      value_1 = ((value_1 * value_1 * (int64_t)CalibrationData._P3) >> 8) + ((value_1 * (int64_t)CalibrationData._P2) << 12);
      value_1 = (((((int64_t)1) << 47) + value_1)) * ((int64_t)CalibrationData._P1) >> 33;

      if (value_1!=0){

        int64_t p = 1048576 - press_raw;
        p = (((p << 31) - value_2) * 3125) / value_1;
        value_1 = (((int64_t)CalibrationData._P9) * (p >> 13) * (p >> 13)) >> 25;
        value_2 = (((int64_t)CalibrationData._P8) * p) >> 19;
        p = ((p + value_1 + value_2) >> 8) + (((int64_t)CalibrationData._P7) << 4);

        pressure = (float)((p/256) * 0.00750061683f);
      }else pressure=0; 
    }else {pressure=0;}
    
    if (h){
      int32_t value  = (temp_raw - ((int32_t)76800));                // Start humidity converting
      value = (((((hum_raw << 14) - (((int32_t)CalibrationData._H4) << 20) -
      (((int32_t)CalibrationData._H5) * value)) +((int32_t)16384)) >> 15) * 
      (((((((value * ((int32_t)CalibrationData._H6)) >> 10) *(((value * 
      ((int32_t)CalibrationData._H3)) >> 11) + ((int32_t)32768))) >> 10) +
      ((int32_t)2097152)) * ((int32_t)CalibrationData._H2) + 8192) >> 14));
      value = (value - (((((value >> 15) * (value >> 15)) >> 7) * ((int32_t)CalibrationData._H1)) >> 4));
      value = (value < 0) ? 0 : value;
      value = (value > 419430400) ? 419430400 : value;
      float h = (value >> 12);

      hum= h / 1024.0;  
    }else{hum=0;}

   rosa = (17.22*temp)/(237.7+temp)+log(hum/100);  
   dew_point=(237.7*rosa)/(17.22-rosa); // Вычисляем точку росы в [*C]
   
  Serial.println(current_mA1);
  Serial.println(current_mA2);
  Serial.println(current_mA3);
  Serial.println(loadvoltage1);
  Serial.println(pressure);
  Serial.println(temp);
  Serial.println(hum);
  Serial.println(dew_point);
    
 //подключение к Wi-Fi  
    File f_config = SPIFFS.open("/config.txt", "r");
  if (!f_config) {
      Serial.println("Error:  File doesn't exist  ");  
      whoami = "0";
      Akey = "NeKKxx1";
      wifi_name = "rt";
      wifi_pass = "apsI0gqvFn";
    } else {
     whoami = f_config.readStringUntil('\n');
     if (whoami.charAt(whoami.length()-1) == 13)
     whoami = whoami.substring(0, whoami.length()-1);
     
     Akey = f_config.readStringUntil('\n');
     if (Akey.charAt(Akey.length()-1) == 13)
     Akey = Akey.substring(0, Akey.length()-1);
     
     wifi_name = f_config.readStringUntil('\n');
     if (wifi_name.charAt(wifi_name.length()-1) == 13)
     wifi_name = wifi_name.substring(0, wifi_name.length()-1);
     
     wifi_pass = f_config.readStringUntil('\n');
     if (wifi_pass.charAt(wifi_pass.length()-1) == 13)
     wifi_pass = wifi_pass.substring(0, wifi_pass.length()-1);
     
     Serial.println("  File exists  ");
    }
  f_config.close();

  WiFi.persistent(false);                 // Не перезаписывать параметры Wi-Fi в EEPROM
  WiFi.disconnect();                      // Отключение станции от точки доступа 
  WiFi.mode(WIFI_OFF); delay(500);        // Установка режима WiFi - Выкл.
  WiFi.mode(WIFI_STA);                    // Установка режима WiFi - «станция»

  //WiFi.setOutputPower(15);                 // Установка мощности передатчика: min=0, max=20.5
  WiFi.setOutputPower(15);

  // Подключение к сети
  WiFi.begin(wifi_name.c_str(), wifi_pass.c_str());

  int c=0;
  
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    c++;
    if (c==25)break;
  }
   Serial.println();
  myIP=WiFi.localIP();                     // Сохранение строки с IP адресом
//  Serial.println("wifi_name");
  Serial.println(myIP);
  Serial.println("");
  sendJsonToDB();

  ESP.deepSleep(600e6);
}

void loop() {}
